'use strict';

var mergeScriptableBanner = require('@scriptables/manifest/mergeScriptableBanner');
var promises = require('fs/promises');
var icloudy = require('icloudy');
var os = require('os');
var path = require('path');
var logger = require('./logger');
var utils = require('./utils');

const findICloudBugsUrl = "https://github.com/icloudy/icloudy/issues";
async function tryReadManifest(filePath, extensions) {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
      const content = await promises.readFile(manifestPath, "utf-8");
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}
function scriptableDeploy(options = {}) {
  const {
    logLevel = "auto",
    continueOnError = true,
    scriptableDir,
    patterns = ["*.{ts,tsx,js,jsx}"],
    addBanner = true,
    manifestExtensions = [".manifest.json", ".manifest", ".json"]
  } = options;
  const filterFile = utils.createPatternMatcher(patterns);
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      const level = logLevel === "auto" ? logger.convertEsbuildLogLevel(build.initialOptions.logLevel || "info") : logLevel;
      const logger$1 = logger.createLogger("scriptable-deploy", { level });
      if (!build.initialOptions.metafile) {
        logger$1.verbose("Automatically enabling metafile option");
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const searchOptions = {
              appName: "Scriptable",
              includeInaccessible: false,
              minScore: 0.5
            };
            const paths = await icloudy.icloudy.find(searchOptions);
            const found = paths && paths.length > 0 ? paths[0] : void 0;
            if (found) {
              if (os.platform() === "darwin" && found.metadata.contents?.includes("Documents")) {
                targetDir = path.join(found.path, "Documents");
              } else {
                targetDir = found.path;
              }
            } else {
              throw new Error("Scriptable directory not found in iCloud Drive");
            }
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          logger$1.verbose(`Target directory: ${targetDir}`);
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger$1.error(message);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          logger$1.error("Missing required configuration");
          return;
        }
        logger$1.time("Deploy Processing");
        const stats = {
          total: 0,
          success: 0,
          skipped: 0,
          failed: 0,
          processedFiles: []
        };
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => {
            const { entryPoint } = result.metafile.outputs[file];
            return entryPoint && filterFile(utils.normalizeAndResolvePath(entryPoint));
          });
          stats.total = outputs.length;
          logger$1.info(`Deploying ${stats.total} files to ${targetDir}...`);
          for (const [index, file] of outputs.entries()) {
            const progress = Math.round((index + 1) / stats.total * 100);
            logger$1.verbose(`[${progress}%] Processing: ${file}`);
            try {
              const output = result.metafile.outputs[file];
              const sourceFile = output.entryPoint;
              if (!sourceFile) {
                logger$1.warn(`No entry point found for ${file}`);
                stats.skipped++;
                stats.processedFiles.push({
                  file,
                  status: "skipped",
                  reason: "No entry point found"
                });
                continue;
              }
              let content;
              try {
                if (result.outputFiles) {
                  const outputFile = result.outputFiles.find((f) => utils.isSamePath(f.path, file));
                  if (!outputFile) {
                    throw new Error(`Output file not found: ${file}`);
                  }
                  content = new TextDecoder().decode(outputFile.contents);
                } else {
                  await utils.waitForFile(file);
                  content = await promises.readFile(file, "utf-8");
                }
              } catch (error) {
                stats.failed++;
                stats.processedFiles.push({
                  file,
                  status: "failed",
                  reason: `Failed to read output file: ${error instanceof Error ? error.message : String(error)}`
                });
                continue;
              }
              let targetFileName = path.basename(file);
              let manifest = null;
              try {
                manifest = await tryReadManifest(sourceFile, manifestExtensions);
                if (manifest?.name) {
                  targetFileName = `${manifest.name}.js`;
                }
              } catch (error) {
                logger$1.warn(`Failed to read manifest for ${file}: ${error}`);
              }
              const targetPath = path.join(targetDir, targetFileName);
              if (addBanner && manifest) {
                try {
                  const { banner: newBanner, content: scriptContent } = mergeScriptableBanner.mergeScriptableBanner(content, manifest);
                  await promises.writeFile(targetPath, newBanner + scriptContent);
                  stats.success++;
                  stats.processedFiles.push({
                    file: `${file} -> ${targetPath}`,
                    status: "success",
                    reason: "Deployed with updated banner"
                  });
                  logger$1.success(`Deployed with updated banner: ${file} -> ${targetPath}`);
                  continue;
                } catch (error) {
                  logger$1.warn(`Failed to add banner to ${file}: ${error}`);
                }
              }
              try {
                await promises.writeFile(targetPath, content);
                stats.success++;
                stats.processedFiles.push({
                  file: `${file} -> ${targetPath}`,
                  status: "success"
                });
                logger$1.success(`Deployed: ${file} -> ${targetPath}`);
              } catch (error) {
                stats.failed++;
                stats.processedFiles.push({
                  file,
                  status: "failed",
                  reason: `Failed to write file: ${error instanceof Error ? error.message : String(error)}`
                });
                logger$1.error(`Failed to deploy ${file}: ${error instanceof Error ? error.message : String(error)}`);
              }
            } catch (error) {
              stats.failed++;
              stats.processedFiles.push({
                file,
                status: "failed",
                reason: error instanceof Error ? error.message : String(error)
              });
              logger$1.error(`Failed to deploy ${file}: ${error instanceof Error ? error.message : String(error)}`);
            }
          }
          logger$1.timeEnd("Deploy Processing");
          logger$1.report(stats);
        } catch (error) {
          logger$1.error(`Deploy failed: ${error.message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    }
  };
}

exports.scriptableDeploy = scriptableDeploy;
//# sourceMappingURL=scriptable-deploy.js.map
//# sourceMappingURL=scriptable-deploy.js.map