import type {ScriptableManifest} from '@scriptables/manifest';
import {mergeScriptableBanner} from '@scriptables/manifest/mergeScriptableBanner';
import type {Plugin} from 'esbuild';
import {readFile, writeFile} from 'fs/promises';
import {icloudy, SearchOptions} from 'icloudy';
import {platform} from 'os';
import {basename, join} from 'path';

import {convertEsbuildLogLevel, createLogger, DetailedStats, LogLevel} from './logger';
import {createPatternMatcher, isSamePath, normalizeAndResolvePath, waitForFile} from './utils';

export interface ScriptableDeployOptions {
  /**
   * Log level for the plugin
   * 'auto' will use esbuild's logLevel
   * Default: 'auto'
   */
  logLevel?: LogLevel | 'auto';

  /**
   * Whether to continue building when deployment fails
   * Default: true
   */
  continueOnError?: boolean;

  /**
   * Custom Scriptable directory path
   * If not specified, will automatically search in iCloud Drive
   */
  scriptableDir?: string;

  /**
   * Source file patterns to match entry points
   * Default: ['**\/*.{ts,tsx,js,jsx}']
   * Examples:
   * -'**\/*.widget.ts'         //only deploy widget files
   * -['**\/*.ts', '!**\/*.test.ts'] //exclude test files
   */
  patterns?: string | string[];

  /**
   * Whether to add Scriptable banner from manifest
   * Default: true
   */
  addBanner?: boolean;

  /**
   * Custom manifest file extensions
   * Default: ['.manifest.json', '.manifest', '.json']
   */
  manifestExtensions?: string[];
}

const findICloudBugsUrl = 'https://github.com/icloudy/icloudy/issues';

async function tryReadManifest(filePath: string, extensions: string[]): Promise<ScriptableManifest | null> {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, '') + ext;
      const content = await readFile(manifestPath, 'utf-8');

      return JSON.parse(content);
    } catch {
      continue;
    }
  }

  return null;
}

export function scriptableDeploy(options: ScriptableDeployOptions = {}): Plugin {
  const {
    logLevel = 'auto',
    continueOnError = true,
    scriptableDir,
    patterns = ['*.{ts,tsx,js,jsx}'],
    addBanner = true,
    manifestExtensions = ['.manifest.json', '.manifest', '.json'],
  } = options;

  const filterFile = createPatternMatcher(patterns);
  let targetDir: string;

  return {
    name: 'scriptable-deploy',
    setup(build) {
      const level = logLevel === 'auto' ? convertEsbuildLogLevel(build.initialOptions.logLevel || 'info') : logLevel;

      const logger = createLogger('scriptable-deploy', {level});

      // Ensure metafile is enabled
      if (!build.initialOptions.metafile) {
        logger.verbose('Automatically enabling metafile option');
        build.initialOptions.metafile = true;
      }

      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            // Find iCloud paths using icloudy
            const searchOptions: SearchOptions = {
              appName: 'Scriptable',
              includeInaccessible: false,
              minScore: 0.5,
            };
            const paths = await icloudy.find(searchOptions);
            const found = paths && paths.length > 0 ? paths[0] : undefined;

            if (found) {
              if (platform() === 'darwin' && found.metadata.contents?.includes('Documents')) {
                targetDir = join(found.path, 'Documents');
              } else {
                targetDir = found.path;
              }
            } else {
              throw new Error('Scriptable directory not found in iCloud Drive');
            }
          }

          if (!targetDir) {
            throw new Error(
              'Scriptable directory not found in iCloud Drive. ' +
                'Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. ' +
                'If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. ' +
                `If the issue persists, please report it at: ${findICloudBugsUrl}`,
            );
          }

          logger.verbose(`Target directory: ${targetDir}`);
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${(error as Error).message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger.error(message);
        }
      });

      build.onEnd(async result => {
        if (!result.metafile || !targetDir) {
          logger.error('Missing required configuration');

          return;
        }

        logger.time('Deploy Processing');

        const stats: DetailedStats = {
          total: 0,
          success: 0,
          skipped: 0,
          failed: 0,
          processedFiles: [],
        };

        try {
          const outputs = Object.keys(result.metafile.outputs).filter(file => {
            const {entryPoint} = result.metafile!.outputs[file];

            return entryPoint && filterFile(normalizeAndResolvePath(entryPoint));
          });

          stats.total = outputs.length;
          logger.info(`Deploying ${stats.total} files to ${targetDir}...`);

          for (const [index, file] of outputs.entries()) {
            const progress = Math.round(((index + 1) / stats.total) * 100);
            logger.verbose(`[${progress}%] Processing: ${file}`);

            try {
              const output = result.metafile.outputs[file];
              const sourceFile = output.entryPoint;

              if (!sourceFile) {
                logger.warn(`No entry point found for ${file}`);
                stats.skipped++;
                stats.processedFiles.push({
                  file,
                  status: 'skipped',
                  reason: 'No entry point found',
                });
                continue;
              }

              // Get file content
              let content: string;
              try {
                if (result.outputFiles) {
                  const outputFile = result.outputFiles.find(f => isSamePath(f.path, file));
                  if (!outputFile) {
                    throw new Error(`Output file not found: ${file}`);
                  }
                  content = new TextDecoder().decode(outputFile.contents);
                } else {
                  await waitForFile(file);
                  content = await readFile(file, 'utf-8');
                }
              } catch (error) {
                stats.failed++;
                stats.processedFiles.push({
                  file,
                  status: 'failed',
                  reason: `Failed to read output file: ${error instanceof Error ? error.message : String(error)}`,
                });
                continue;
              }

              // Try to read the manifest using entryPoint
              let targetFileName = basename(file);
              let manifest: ScriptableManifest | null = null;

              try {
                manifest = await tryReadManifest(sourceFile, manifestExtensions);
                if (manifest?.name) {
                  targetFileName = `${manifest.name}.js`;
                }
              } catch (error) {
                logger.warn(`Failed to read manifest for ${file}: ${error}`);
              }

              const targetPath = join(targetDir, targetFileName);

              if (addBanner && manifest) {
                try {
                  const {banner: newBanner, content: scriptContent} = mergeScriptableBanner(content, manifest);
                  await writeFile(targetPath, newBanner + scriptContent);
                  stats.success++;
                  stats.processedFiles.push({
                    file: `${file} -> ${targetPath}`,
                    status: 'success',
                    reason: 'Deployed with updated banner',
                  });
                  logger.success(`Deployed with updated banner: ${file} -> ${targetPath}`);
                  continue;
                } catch (error) {
                  logger.warn(`Failed to add banner to ${file}: ${error}`);
                }
              }

              // If the banner is not needed or the processing fails, write the content directly
              try {
                await writeFile(targetPath, content);
                stats.success++;
                stats.processedFiles.push({
                  file: `${file} -> ${targetPath}`,
                  status: 'success',
                });
                logger.success(`Deployed: ${file} -> ${targetPath}`);
              } catch (error) {
                stats.failed++;
                stats.processedFiles.push({
                  file,
                  status: 'failed',
                  reason: `Failed to write file: ${error instanceof Error ? error.message : String(error)}`,
                });
                logger.error(`Failed to deploy ${file}: ${error instanceof Error ? error.message : String(error)}`);
              }
            } catch (error) {
              stats.failed++;
              stats.processedFiles.push({
                file,
                status: 'failed',
                reason: error instanceof Error ? error.message : String(error),
              });
              logger.error(`Failed to deploy ${file}: ${error instanceof Error ? error.message : String(error)}`);
            }
          }

          logger.timeEnd('Deploy Processing');
          logger.report(stats);
        } catch (error) {
          logger.error(`Deploy failed: ${(error as Error).message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    },
  };
}
